//+-------------------------------------------------------------------------------------+
//|                                                             SessionTrade_Expert.mq4 |
//|                                                                           Scriptong |
//|                                                                   scriptong@mail.ru |
//+-------------------------------------------------------------------------------------+
#property copyright "Scriptong"
#property link      "scriptong@mail.ru"

//---- input parameters
extern double    Lots              = 0.1;
extern int       Delta             = 1;
extern int       AzianFlatStart    = 0;
extern int       AzianFlatEnd      = 8;
extern int       EuropeanFlatStart = 8;
extern int       EuropeanFlatEnd   = 16;
extern int       AmericanFlatStart = 16;
extern int       AmericanFlatEnd   = 24;
extern string    OpenOrderSound    = "ok.wav";
extern int       MagicNumber       = 47589;

bool Activate, FreeMarginAlert, FatalError;
double Tick, Spread, StopLevel, MinLot, MaxLot, LotStep, LastLots, 
       LowV, HighV, SessionLow, SessionHigh;
datetime LastBar, NowDay, LastOpen[3];
int Start[3], Finish[3], CurSession;

//+-------------------------------------------------------------------------------------+
//| expert initialization function                                                      |
//+-------------------------------------------------------------------------------------+
int init()
  {
//----
   Activate = False;
   FatalError = False;

// - 1 - ==      ========================================   
   Tick = MarketInfo(Symbol(), MODE_TICKSIZE);                         //      
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                 //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //    
   MinLot = MarketInfo(Symbol(), MODE_MINLOT);    //    
   MaxLot = MarketInfo(Symbol(), MODE_MAXLOT);   //    
   LotStep = MarketInfo(Symbol(), MODE_LOTSTEP);          //    
// - 1 - ==   =============================================================

// - 2 - ==          =======   
   Lots = MathRound(Lots/LotStep)*LotStep; //     
   if(Lots < MinLot || Lots > MaxLot) //     MinLot    MaxLot
     {
      Comment(" Lots     !  !");
      return(0);
     }
// - 2 - ==   =============================================================
    
// - 3 - ===          =====
   if (!ToArray(0, AzianFlatStart, AzianFlatEnd) || 
       !ToArray(1, EuropeanFlatStart, EuropeanFlatEnd) ||
       !ToArray(2, AmericanFlatStart, AmericanFlatEnd))
     return(0);  
// - 3 - ===   ============================================================

   ArrayInitialize(LastOpen, 0);

// - 4 - ===     LastOpen =========================
   for (int i = OrdersHistoryTotal() - 1; i >= 0; i--)    
     if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY))            //    
       if (OrderSymbol() == Symbol() && MathFloor(OrderMagicNumber()/10) == MagicNumber)
         {
          int ID = MathMod(OrderMagicNumber(), 10);
          LastOpen[ID] = MathMax(LastOpen[ID], OrderOpenTime());
         }
         
   for (i = OrdersTotal() - 1; i >= 0; i--)    
     if (OrderSelect(i, SELECT_BY_POS))                         //    
       if (OrderSymbol() == Symbol() && MathFloor(OrderMagicNumber()/10) == MagicNumber)
         {
          ID = MathMod(OrderMagicNumber(), 10);
          LastOpen[ID] = MathMax(LastOpen[ID], OrderOpenTime());
         }
// - 4 - ===   ============================================================

   LastBar = 0;
   Activate = True;
   
//----
   return(0);
  }
//+-------------------------------------------------------------------------------------+
//| expert deinitialization function                                                    |
//+-------------------------------------------------------------------------------------+
int deinit()
  {
//----
   Comment("");   
//----
   return(0);
  }

//+-------------------------------------------------------------------------------------+
//|                                               |
//+-------------------------------------------------------------------------------------+
bool ToArray(int Num, int St, int End)
{
 if (St < 0 || End < 0 || St > 24 || End > 24 || St >= End)
     {
      Comment("  Start  End      0  24  Start < End.");
      Print("  Start  End      0  24  Start < End.");
      return(False);
     } 
 int i = Num-1;    
 while (i >= 0)
   {
    if (Start[i] >= St || Finish[i] >= End || Finish[i] > St)
      {
       Comment("  .  .");
       Print("  .  .");
       return(False);
      }
    i--;
   }    
   
 Start[Num] = St;
 Finish[Num] = End;
 return(True);  
}


//+------------------------------------------------------------------------------------+
//|                                               |
//+------------------------------------------------------------------------------------+
double ND(double A)
{
 return(NormalizeDouble(A, Digits));
}  

//+-------------------------------------------------------------------------------------+
//|                                                                   |
//+-------------------------------------------------------------------------------------+
void GetSignal()
{
// - 1 - ====     ,      =========
 LowV = 0;
 HighV = 0;
 CurSession = -1;
 int DayBar = 0; 
 int k = 0;
// - 1 - =============================   ==================================

// - 2 - ====     =======================================
 for (k = 0; k < 3; k++)
   if (Hour() >= Start[k] && Hour() < Finish[k])
     {
      if (k == 0)
        {
         DayBar++;
         int j = 2;
        } 
       else
        j = k - 1; 
// - 2 - =============================   ==================================

// - 3 - ====   ============================================================
      datetime BeginDay = iTime(Symbol(), PERIOD_D1, DayBar);         //   
      if (!(TimeDayOfWeek(BeginDay) == 5 && k == 0))        //   
        {                                                                 // 
         // ,    
         int StartBar = iBarShift(Symbol(), 0, BeginDay+Start[j]*3600);
         // ,    
         int FinishBar = iBarShift(Symbol(), 0, BeginDay+Finish[j]*3600)+1; 
         //  
         LowV = Low[iLowest(Symbol(), 0, MODE_LOW, StartBar-FinishBar+1, FinishBar)];
         //    
         HighV = High[iHighest(Symbol(), 0, MODE_HIGH, StartBar-FinishBar+1, FinishBar)]; 
         //   
         SessionLow =  Low[iLowest(Symbol(), 0, MODE_LOW, FinishBar+1)]; 
         //   
         SessionHigh = High[iHighest(Symbol(), 0, MODE_HIGH, FinishBar+1)];
        }
      CurSession = k;//    (0-, 1-, 2-)
      break;
     } 
// - 3 - =============================   ==================================
}  

//+-------------------------------------------------------------------------------------+
//|                                                         |
//+-------------------------------------------------------------------------------------+
string ErrorToString(int Error)
{
 switch(Error)
   {
    case 2: return("  ,   ."); 
    case 5: return("    ,  ."); 
    case 6: return("   ,   ."); 
    case 64: return(" ,   .");
    case 132: return(" ."); 
    case 133: return(" ."); 
    case 149: return(" ."); 
   }
}
  
//+-------------------------------------------------------------------------------------+
//| ""                                                         |
//|    OpenOrder            |
//| :                                                                         |
//|   0 -                                                                      |
//|   1 -                                                                 |
//|   2 -   Price                                                         |
//|   3 -   SL                                                            |
//|   4 -   TP                                                            |
//|   5 -   Lot                                                           |
//+-------------------------------------------------------------------------------------+
int OpenOrderCorrect(int Type, double Lot, double Price, double SL, double TP,
                     bool Redefinition = True)
// Redefinition -  True     
//                 False -  
{
// - 1 - ==     ====================================
 if(AccountFreeMarginCheck(Symbol(), OP_BUY, Lot) <= 0 || GetLastError() == 134) 
  {
   if(!FreeMarginAlert)
    {
     Print("    . Free Margin = ", 
           AccountFreeMargin());
     FreeMarginAlert = True;
    } 
   return(5);  
  }
 FreeMarginAlert = False;  
// - 1 - ==   =============================================================

// - 2 - ==   Price, SL  TP    ====================   

 RefreshRates();
 switch (Type)
   {
    case OP_BUY: 
                string S = "BUY"; 
                if (MathAbs(Price-Ask)/Point > 3)
                  if (Redefinition) Price = ND(Ask);
                  else              return(2);
                if (ND(TP-Bid) <= StopLevel && TP != 0)
                  if (Redefinition) TP = ND(Bid+StopLevel+Tick);
                  else              return(4);
                if (ND(Bid-SL) <= StopLevel)
                  if (Redefinition) SL = ND(Bid-StopLevel-Tick);
                  else              return(3);
                break;
    case OP_SELL: 
                 S = "SELL"; 
                 if (MathAbs(Price-Bid)/Point > 3)
                   if (Redefinition) Price = ND(Bid);
                   else              return(2);
                 if (ND(Ask-TP) <= StopLevel) 
                   if (Redefinition) TP = ND(Ask-StopLevel-Tick);
                   else              return(4);
                 if (ND(SL-Ask) <= StopLevel && SL != 0)
                   if (Redefinition) SL = ND(Ask+StopLevel+Tick);
                   else              return(3);
                 break;
    case OP_BUYSTOP: 
                    S = "BUYSTOP";
                    if (ND(Price-Ask) <= StopLevel)
                      if (Redefinition) Price = ND(Ask+StopLevel+Tick);
                      else              return(2);
                    if (ND(TP-Price) <= StopLevel && TP != 0)
                      if (Redefinition) TP = ND(Price+StopLevel+Tick);
                      else              return(4);
                    if (ND(Price-SL) <= StopLevel)
                      if (Redefinition) SL = ND(Price-StopLevel-Tick);
                      else              return(3);
                    break;
    case OP_SELLSTOP: 
                     S = "SELLSTOP";
                     if (ND(Bid-Price) <= StopLevel)
                       if (Redefinition) Price = ND(Bid-StopLevel-Tick);
                       else              return(2);
                     if (ND(Price-TP) <= StopLevel)
                       if (Redefinition) TP = ND(Price-StopLevel-Tick);
                       else              return(4);
                     if (ND(SL-Price) <= StopLevel && SL != 0)
                       if (Redefinition) SL = ND(Price+StopLevel+Tick);
                       else              return(3);
                     break;
    case OP_BUYLIMIT: 
                     S = "BUYLIMIT";
                     if (ND(Ask-Price) <= StopLevel)
                      if (Redefinition) Price = ND(Ask-StopLevel-Tick);
                      else              return(2);
                     if (ND(TP-Price) <= StopLevel && TP != 0)
                       if (Redefinition) TP = ND(Price+StopLevel+Tick);
                       else              return(4);
                     if (ND(Price-SL) <= StopLevel)
                       if (Redefinition) SL = ND(Price-StopLevel-Tick);
                       else              return(3);
                     break;
    case OP_SELLLIMIT: 
                     S = "SELLLIMIT";
                     if (ND(Price - Bid) <= StopLevel) 
                       if (Redefinition) Price = ND(Bid+StopLevel+Tick);
                       else              return(2);
                     if (ND(Price-TP) <= StopLevel)
                       if (Redefinition) TP = ND(Price-StopLevel-Tick);
                       else              return(4);
                     if (ND(SL-Price) <= StopLevel && SL != 0)
                       if (Redefinition) SL = ND(Price+StopLevel+Tick);
                       else              return(3);
                     break;
   }
// - 2 - ==   =============================================================
 
// - 3 - ==       =================================   
 if(WaitForTradeContext())  //    
   {  
    Comment("     ", S, " ...");
    datetime Exp = NowDay + Finish[CurSession]*3600;
    if (Exp - TimeCurrent() <= 600)
      Exp = TimeCurrent() + 660;  
    int ticket=OrderSend(Symbol(), Type, Lot, Price, 3, 
               SL, TP, NULL, MagicNumber*10+CurSession, Exp);//  
    //     
    if(ticket<0)
      {
       int Error = GetLastError();
       if(Error == 2 || Error == 5 || Error == 6 || Error == 64 
          || Error == 132 || Error == 133 || Error == 149)     //   
         {
          Comment("     . . "+
                   ErrorToString(Error)+"  !");
          FatalError = True;
         }
        else 
         Comment("   ", S, ": ", Error);       //  
       return(1);
      }
    // ---------------------------------------------
    
    //      
    Comment(" ", S, "  !"); 
    PlaySound(OpenOrderSound); 
    return(0); 
    // ------------------------
   }
  else
   {
    Comment("     !");
    return(1);  
   } 
// - 3 - ==   =============================================================  
}

//+-----------------------------------------------------------------------------------+
//|   .   ,   True,  - False  |
//+-----------------------------------------------------------------------------------+  
bool WaitForTradeContext()
{
 int P = 0;
 //  ""
 while(IsTradeContextBusy() && P < 5)
   {
    P++;
    Sleep(1000);
   }
 // -------------  
 if(P == 5)
   return(False);
 return(True);    
}

//+-------------------------------------------------------------------------------------+
//|                                                           |
//+-------------------------------------------------------------------------------------+
void DoTransactions()
{
// - 1 - ==   Buy Stop ===================================================
 if (LastOpen[CurSession] < NowDay)   //       ?
   if (Close[1] < LowV && Open[1] < LowV)   //      
     {
      RefreshRates();
      double Price = ND(LowV + Spread + Delta*Tick);   //   Buy Stop - 
                                                                        //  
      double SL = ND(SessionLow - (HighV - LowV) - Tick);            //   Buy Stop
      double TP = ND(HighV);      //   Buy Stop -   
      if (OpenOrderCorrect(OP_BUYSTOP, Lots, Price, SL, TP, False) == 0)
        LastOpen[CurSession] = TimeCurrent();   //   ,  
       else 
        return;
     }    
// - 1 - ===   ============================================================

// - 2 - ===   Sell Stop =================================================
 if (LastOpen[CurSession] < NowDay)   //       ?
   if (Close[1] > HighV && Open[1] > HighV)//      
     {
      RefreshRates();
      Price = ND(HighV);              //   Sell Stop -   
      SL = ND(SessionHigh + (HighV - LowV) + Spread + Tick);        //   Sell Stop
      TP = ND(LowV + Spread);    //   Sell Stop -   
      if (OpenOrderCorrect(OP_SELLSTOP, Lots, Price, SL, TP, False) == 0)
        LastOpen[CurSession] = TimeCurrent();   //   ,  
       else 
        return;
     }    
// - 2 - ===   ============================================================
   
 LastBar = Time[0];                                 //      
}
  
//+-------------------------------------------------------------------------------------+
//|  START                                                               |
//+-------------------------------------------------------------------------------------+
int start()
  {
// - 1 - ===    ? ===========================================
   if (!Activate || FatalError)             //   ,  
     return(0);          //  init         
// - 1 - ===   ============================================================

// - 2 - ==         ==============
   if (LastBar == Time[0])
     return(0);
// - 2 - ===   ============================================================
     
// - 3 - ==      ========================================
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                 //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //    
// - 3 - ===   ============================================================
   
// - 4 - ==         ====================
   NowDay = iTime(Symbol(), PERIOD_D1, 0);
   GetSignal();
// - 4 - ===   ============================================================

// - 5 - ==   ,    ========================
   if (LowV != 0 && HighV != 0 && CurSession != -1)
      DoTransactions();
// - 5 - ===   ============================================================

//----
   return(0);
  }
//+------------------------------------------------------------------+